library(shiny)

# Define UI for application that draws a histogram
ui <- fluidPage(
  
  ## Title line
  titlePanel("Example of Management Strategy Evaluation"),
  
  ## First line
  fluidRow(
    column(6, wellPanel(
      textInput("Objectives", h4("What is your management objective?"), value="Enter your objectives"),
      numericInput(inputId = "Objective1", label = "Prob(B > Bmsy) = ", value = 0.5, step=0.1, min=0, max=1)
    )), 
    column(6, wellPanel(
      numericInput(inputId = "Nyear", label = "Management Period:", value = 10, step=1, width = "50%"),
      numericInput(inputId = "Nsim", label = "Number of simulation replicas:", value = 500, max=1000, step=100, width = "50%")
    )) 
  ),
  
  ## Second line
  fluidRow(
    
    column(3, wellPanel(
      h4("Configulation of Operating Models"), 
      numericInput(inputId = "Dinitial", label = "Initial Depletion (D0):", value = 0.4, step=0.05, min=0.1, max=0.9),
      numericInput(inputId = "r", label = "Intrinsic Rate of Increase (r):", value = 0.1, step=0.01, min=0.01, max=0.5),
      numericInput(inputId = "K", label = "Carrying Capacity (K):", value = 10000, step=1000),
      numericInput(inputId = "z", label = "Shape Parameter (z):", value = 1, step=0.1, min=0.1, max=3),
      numericInput(inputId = "EstCV", label = "Precision of Abundance Estimate (CV):", value = 0.1, step=0.05, min=0, max=0.5), 
      numericInput(inputId = "ProCV", label = "Extent of Process Error (tau):", value = 0.05, step=0.01, min=0, max=0.2)
    )),
    
    column(3, wellPanel(
      h4("Setting your Management Procedure"), 
      numericInput(inputId = "LRP", label = "Limit Reference Point:", value = 0.1, step=0.05, min=0, max=0.5),
      numericInput(inputId = "TRP", label = "Target Reference Point:", value = 0.4, step=0.05),
      numericInput(inputId = "MaxChange", label = "Maximum Allowable Change in Catch (%):", value = 30, step=5),
      selectInput(inputId = "Assessment", label = "Do assessment? (this does not work yet):", 
                  choices = list("Yes, parameters are updated"=1, "No, assessment is not conducted"=2), selected=1),
      submitButton("Run"),
      downloadButton("report", "Download PDF")
    )),
    
    column(6, wellPanel(
      h4("Simulation results"), 
      tabsetPanel(
        tabPanel("Performance Mesures",tableOutput("Performance")),
        tabPanel("Your Harvest Control Rule", plotOutput("HCR")),
        tabPanel("Kobe Matrix", tableOutput("Kobe2"))
      )
    ))
  ), 
  
  ## Third line 
  fluidRow(
    column(4,wellPanel(
      plotOutput("Trajectories_B")
    )),
    
    column(4,wellPanel(
      plotOutput("Trajectories_F")
    )),
    
    column(4,wellPanel(
      plotOutput("Kobe1")
    ))
  )
  
)

# Define server logic required to draw a histogram
server <- function(input, output){
  
  Simulation <- reactive({
    
    #Model-based
    HCR <- function(Bcurrent, Dep, rest, Kest, zest, LRP, TRP)
    {
      Fmsy <- rest*(1/(1+zest))^(1/zest)
      if(TRP > LRP){
        if(Dep <= LRP) Quota <- 0
        if(Dep > LRP & Dep < TRP) Quota <- (Fmsy/(TRP-LRP))*(Dep-LRP)*Bcurrent
        if(Dep >= TRP) Quota <- Fmsy*Bcurrent
      }
      if(TRP-LRP==0){
       if(Dep <= LRP) Quota <- 0
      if(Dep >= LRP) Quota <- Fmsy*Bcurrent             
      }
      return(Quota)
    }
    
    #Pella-Tomlinson Production model (incl. Shaefer model)
    PDM.PT <- function(r, K, z, Dinitial, ProCV, Nyear)
    {
      Pmsy <- K*(1/(1+z))^(1/z)
      Fmsy <<- r*(z/(1+z))
      P <- numeric()
      Catch <- numeric()
      Bratio <- numeric()
      Fratio <- numeric()
      ProError <- rnorm(Nyear,0,ProCV)
      P[1] <- K*Dinitial
      for(t in 1:Nyear)
      {
        Pest <- P[t]*exp(rnorm(1,0,EstCV))
        Catch[t] <- HCR(Pest, Pest/K, r, K, z, LRP, TRP)
        if(t>=2){ 
          if(Catch[t-1]>0) change <- (Catch[t] - Catch[t-1])/Catch[t-1] 
          if(Catch[t-1]==0) change <- 0
          if(change > MaxChange/100) Catch[t] <- (1+MaxChange/100)*Catch[t-1]
          if(change < -MaxChange/100) Catch[t] <- (1-MaxChange/100)*Catch[t-1]
        }
        tmp <- P[t] + r*P[t]*(1-(P[t]/K)^z) - Catch[t]
        P[t+1] <- max(tmp*exp(ProError[t]),1)
      }
      Out <- NULL
      Out$P <- P
      Out$Bratio <- P/Pmsy
      Out$Fratio <- Catch/(Fmsy*Pmsy)
      Out$Catch <- Catch
      return(Out)
    }
    
    Nyear <<- input$Nyear
    Dinitial <<- input$Dinitial
    r <<- input$r
    K <<- input$K
    z <<- input$z
    ProCV <<- input$ProCV
    EstCV <<- input$EstCV
    LRP <<- input$LRP
    TRP <<- input$TRP
    MaxChange <<- input$MaxChange
    Nsim <<- input$Nsim
    
    P.mat <- array(0,c(Nsim, Nyear+1))
    Bratio.mat <- array(0,c(Nsim, Nyear+1))
    Catch.mat <- array(0,c(Nsim, Nyear))
    Fratio.mat <- array(0,c(Nsim, Nyear))
    for(i in 1:Nsim){
      Res <- PDM.PT(r,K,z,Dinitial,ProCV,Nyear)
      P.mat[i,] <- Res$P
      Bratio.mat[i,] <- Res$Bratio
      Fratio.mat[i,] <- Res$Fratio
      Catch.mat[i,] <- Res$Catch
    }
    SimRes<-NULL
    SimRes$P.mat <- P.mat
    SimRes$Bratio.mat <- Bratio.mat
    SimRes$Catch.mat <- Catch.mat
    SimRes$Fratio.mat <- Fratio.mat
    SimRes$P.med <- apply(P.mat,2,median)
    SimRes$Bratio.med <- apply(Bratio.mat,2,median)
    SimRes$Fratio.med <- apply(Fratio.mat,2,median)
    return(SimRes)
    
  })
  
  output$Trajectories_B <- renderPlot({
    
    SimRes <- Simulation()
    par(mfrow=c(3,1), mar=c(2,2,2,2))
    
    plot(0,0, main="Population size", xlim=c(0,Nyear), ylim=c(0,K*1.1), xaxs="i", yaxs="i")
    for(i in 1:Nsim) points(0:Nyear, SimRes$P.mat[i,], type="l", col="lightgray")
    abline(K,0,lty=2)
    points(0:Nyear, SimRes$P.med, type="l", col="blue", lwd=2)
    
    plot(0,0, main="Depletion level", xlim=c(0,Nyear), ylim=c(0,1.1), xaxs="i", yaxs="i")
    for(i in 1:Nsim) points(0:Nyear, SimRes$P.mat[i,]/K, type="l", col="lightgray")
    points(0:Nyear, SimRes$P.med/K, type="l", col="blue", lwd=2)
    rect(-100,0,100,LRP, col=rgb(1, 0, 0, alpha=0.2))
    rect(-100,LRP,100,TRP, col=rgb(1, 1, 0, alpha=0.2))
    rect(-100,TRP,100,10, col=rgb(0, 1, 0, alpha=0.2))
    
    plot(0,0, main="B-ratio", xlim=c(0,Nyear), ylim=c(0,max(c(max(SimRes$Bratio.mat)*1.1,2))), xaxs="i", yaxs="i")
    for(i in 1:Nsim)  points(0:Nyear, SimRes$Bratio.mat[i,], type="l", col="lightgray")
    points(0:Nyear, SimRes$Bratio.med, type="l", col="blue", lwd=2)
    rect(-100,0,100,1, col=rgb(1, 0, 0, alpha=0.2))
    rect(-100,1,100,100, col=rgb(0, 1, 0, alpha=0.2))
    
  })
  
  output$Trajectories_F <- renderPlot({
    
    SimRes <- Simulation()
    par(mfrow=c(3,1), mar=c(2,2,2,2))
    
    plot(0,0,main="Catch", xlim=c(0,Nyear), ylim=c(0,max(SimRes$Catch)*1.1), xaxs="i", yaxs="i")
    for(i in 1:Nsim) points(1:Nyear, SimRes$Catch[i,], type="l", col="lightgray")
    Catch.med <- apply(SimRes$Catch.mat,2,median)
    points(1:Nyear, Catch.med, type="l", col="blue", lwd=2)
    
    plot(0,0,main="Change in catch (%)", xlim=c(0,Nyear), ylim=c(-100,100), xaxs="i", yaxs="i")
    for(i in 1:Nsim) points(1:(Nyear-1), 100*(SimRes$Catch[i,-1]-SimRes$Catch[i,-Nyear])/SimRes$Catch[i,-Nyear], 
                            type="l", col="lightgray")
    abline(MaxChange, 0, lty=2, col="blue")
    abline(0, 0, lty=2, col="gray")
    abline(-MaxChange, 0, lty=2, col="blue")
    
    plot(0,0,main="F-ratio", xlim=c(0,Nyear), ylim=c(0,max(c(max(SimRes$Fratio.mat)*1.1, 2))), xaxs="i", yaxs="i")
    for(i in 1:Nsim) points(1:Nyear, SimRes$Fratio.mat[i,], type="l", col="lightgray")
    points(1:Nyear, SimRes$Fratio.med, type="l", col="blue", lwd=2)
    rect(-100,0,100,1, col=rgb(0, 1, 0, alpha=0.2))
    rect(-100,1,100,100, col=rgb(1, 0, 0, alpha=0.2))
    
  })
  
  output$Kobe1 <- renderPlot({
    
    SimRes <- Simulation()    
    par(mfrow=c(1,1), xaxs="i", yaxs="i")
    
    Bratio.med <- SimRes$Bratio.med[-(Nyear+1)]
    Fratio.med <- SimRes$Fratio.med
    
    xmax<-max(Bratio.med*1.05,2)
    ymax<-max(Fratio.med*1.05,2)
    plot(Bratio.med, Fratio.med, xlab="B-ratio", ylab="F-ratio",xlim=c(0,xmax),ylim=c(0,ymax),col="red")
    rect(0,1,1,ymax,col="lightsalmon")
    rect(0,0,1,1,col="lightyellow")
    rect(1,1,xmax,ymax,col="lightyellow")
    rect(1,0,xmax,1,col="lightgreen")
    points(SimRes$Bratio.mat[,Nyear],SimRes$Fratio.mat[,Nyear], pch=19, cex=0.5, col="gray")
    lines(Bratio.med,Fratio.med,type="o")
    
  })
  
  output$Performance <- renderTable({
    SimRes <- Simulation()   
    Catch.med <- apply(SimRes$Catch.mat,2,median)
    judge <- function(x) mean(x>1)
    Prob1 <- apply(SimRes$Bratio.mat[,-1], 2, judge)
    Prob2 <- apply(SimRes$Fratio.mat, 2, judge)
    tmp<- data.frame(Year=1:Nyear, B.med=SimRes$P.med[-1], Catch.med=Catch.med, Bratio.med=SimRes$Bratio.med[-1], 
                       Fratio.med=SimRes$Fratio.med, "Prob B greater than Bmsy="= Prob1, "Prob F smaller than Fmsy" = 1-Prob2)
    return(tmp)
  })
  
  output$HCR <- renderPlot({
    SimRes <- Simulation()
    plot(0,0,type="n",xlim=c(0,1), ylim=c(0,Fmsy*1.5), xlab="Depeletion level", ylab="Fishing Intensity")
    lines(c(0,LRP),c(0,0),lwd=2)
    lines(c(LRP,TRP),c(0,Fmsy),lwd=2)
    lines(c(TRP,1),c(Fmsy,Fmsy),lwd=2)
    lines(c(TRP,TRP),c(0,Fmsy),lty=2)
    lines(c(0,TRP),c(Fmsy,Fmsy),lty=2)
  })
  
  output$report <- downloadHandler(
    
    filename = function(){
      paste("MSE",format(Sys.time(), "%d-%b-%Y %H.%M"), ".pdf", sep=".")
    },
    content = function(file){
      pdf(file)
      hist(rnorm(10000,0,1), main="仮の入れ物")
      dev.off()
    }
  )

}

# Run the application 
shinyApp(ui = ui, server = server)

